{
 This demo shows how to create a table from 3 RichViewEdits.
 In this demo, all source RichViewEdits are linked to their own RVStyles:
 - RichViewEdit1.Style = RVStyle1
 - RichViewEdit2.Style = RVStyle2
 - RichViewEdit3.Style = RVStyle3.
 The destination RichView (rvTable) is linked to its own RVStyle (RVStyle4).
 Documents are copied via RVF stream.

 This demo shows how to solve the problems with cells: they do not support
 loading RVF data with text, paragraph and list styles. To solve this problem,
 a hidden TRichView (rvHidden) is used. Important: it is linked to the same
 RVStyle as rvTable (RVStyle4)
}

unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, RichView, RVStyle, RVScroll, RVEdit, RVTable, StdCtrls;

type
  TForm1 = class(TForm)
    RichViewEdit1: TRichViewEdit;
    RVStyle1: TRVStyle;
    RichViewEdit2: TRichViewEdit;
    RVStyle2: TRVStyle;
    RichViewEdit3: TRichViewEdit;
    RVStyle3: TRVStyle;
    rvTable: TRichView;
    rvHidden: TRichView;
    RVStyle4: TRVStyle;
    Button1: TButton;
    procedure Button1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
    procedure CopyToCell(rv: TCustomRichView; Cell: TRVTableCellData);
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

// Copying rv to Cell
procedure TForm1.CopyToCell(rv: TCustomRichView; Cell: TRVTableCellData);
var Stream: TMemoryStream;
    UnusedColor: TColor;
begin
  // We copy from rv to Cell via Stream in RVF format.

  // Unfortunately, we cannot copy rv to Cell directly, because
  // the Cell's TRichView component (rvTable) and rv are linked to different
  // TRVStyle components. Cells do not support loading RVF containing styles.

  // So we copy in 2 steps:
  // 1) from rv to rvHidden (with styles). rvHidden is linked to the same
  //   TRVStyle as rvTable.
  Stream := TMemoryStream.Create;
  rv.SaveRVFToStream(Stream, False);
  Stream.Position := 0;
  rvHidden.Clear;
    // we cannot use LoadRVFFromStream, because LoadRVFFromStream replaces styles,
    // but we need to merge styles. So we use InsertRVFFromStream.
  rvHidden.InsertRVFFromStream(Stream, 0);


  // 2) from rvHidden to Cell (without styles)
  rvHidden.RVFOptions := rvHidden.RVFOptions - [rvfoSaveTextStyles, rvfoSaveParaStyles];
  Stream.Clear;
  rvHidden.SaveRVFToStream(Stream, False);
  Stream.Position := 0;
  Cell.LoadRVFFromStream(Stream, UnusedColor, nil, nil, nil, nil);
end;

procedure TForm1.Button1Click(Sender: TObject);
var table: TRVTableItemInfo;
begin
  rvTable.Clear;
  rvTable.DeleteUnusedStyles(True, True, True);
  table := TRVTableItemInfo.CreateEx(1, 3, rvTable.RVData);
  table.BorderWidth := 3;
  table.CellBorderWidth := 1;
  CopyToCell(RichViewEdit1, table.Cells[0, 0]);
  CopyToCell(RichViewEdit2, table.Cells[0, 1]);
  CopyToCell(RichViewEdit3, table.Cells[0, 2]);
  rvTable.AddItem('', table);
  rvTable.Format;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  RichViewEdit1.Clear;
  RichViewEdit1.AddNL('Text 1', 0, 0);
  RichViewEdit1.Format;

  RichViewEdit2.Clear;
  RichViewEdit2.AddNL('Text 2', 0, 0);
  RichViewEdit2.Format;

  RichViewEdit3.Clear;
  RichViewEdit3.AddNL('Text 3', 0, 0);
  RichViewEdit3.Format;
end;

end.
